@@ -12,6 +12,15 @@ def _render_from_string(templatestring, context=None):
     template = Template(templatestring)
     return template.render(Context(context))
 
+class MailrobotError(ValueError):
+    pass
+
+class MailrobotNoSenderError(MailrobotError):
+    pass
+
+class MailrobotNoRecipientsError(MailrobotError):
+    pass
+
 class NameManager(models.Manager):
     def get_by_natural_key(self, name):
         return self.get(name=name)
@@ -69,10 +78,27 @@ class Signature(AbstractNamedModel):
     def __unicode__(self):
         return self.name
 
+    def attach(self, context=None):
+        """
+        Signature is attached so::
+
+                the final line of content
+
+
+                --
+                signature
+
+        """
+
+        signature = _render_from_string(self.sig, context)
+        if signature:
+            return u'\r\n\r\n\r\n-- \r\n%s' % signature
+        return u''
+
 class MailBody(AbstractNamedModel):
     "Subject and bodytext of the email"
 
-    subject = models.CharField(max_length=66, unique=True)
+    subject = models.CharField(max_length=66)
     body = models.TextField()
 
     class Meta:
@@ -124,22 +150,10 @@ class Mail(AbstractNamedModel):
         return self.content.body
 
     def attach_signature(self, context=None):
-        """
-        Signature is attached so:
-
-        ::
-
-                the final line of content
-
-
-                --
-                signature
-
-        """
+        "Attach signature, if any"
 
         if self.signature:
-            signature = _render_from_string(self.signature.sig, context)
-            return u'\n\n\n-- \n%s' % signature
+            return self.signature.attach(context)
         return u''
 
     def make_content(self, context=None):
@@ -168,7 +182,7 @@ class Mail(AbstractNamedModel):
                 return unicode(self.sender)
         else:
             return sender
-        raise ValueError, "Mail must have a sender"
+        raise MailrobotNoSenderError, "Mail must have a sender"
 
     def _get_addresses(self, attribute, additional=(), required=False):
         "Merge the addresses of field <attribute> with list in <additional>"
@@ -179,18 +193,41 @@ class Mail(AbstractNamedModel):
             addresses = set([unicode(row) for row in attribute.all()])
             addresses = addresses | set(additional)
         if required and not addresses:
-            raise ValueError, 'No addresses!'
+            raise MailrobotNoRecipientsError, 'No recipient addresses!'
         return addresses
 
-    def get_recipients(self, additional=()):
-        return self._get_addresses('recipients', additional, required=True)
+    def get_recipients(self, additional=(), required=True):
+        """Get recipients for To: and ensures there is at least one.
+
+        Email lacking anything in To: is likely spam."""
+
+        return self._get_addresses('recipients', additional, required)
 
     def get_ccs(self, additional=()):
+        """Get recipients for CC:
+
+        May be empty."""
+
         return self._get_addresses('ccs', additional)
 
     def get_bccs(self, additional=()):
+        """Get recipients for BCC:
+
+        May be empty."""
+
         return self._get_addresses('bccs', additional)
 
+    def validate_addresses(self, sender=None, recipients=(), ccs=(), bccs=()):
+        sender = self.get_sender(sender)
+        recipients = self.get_recipients(recipients, required=False)
+        ccs = self.get_ccs(ccs)
+        bccs = self.get_bccs(bccs)
+        if not sender:
+            raise MailrobotNoSenderError, "Invalid Email: Lacks sender"
+        if not (recipients | ccs | bccs):
+            raise MailrobotNoRecipientsError, "Invalid Email: Has no recipients"
+        return True
+
     def make_message(self, sender=None, recipients=(), ccs=(), bccs=(), reply_to=None, headers=None, context=None):
         """Generate a django.core.mail.EmailMessage
 
@@ -208,6 +245,8 @@ class Mail(AbstractNamedModel):
         ccs = self.get_ccs(ccs)
         bccs = self.get_bccs(bccs)
 
+        sendable = self.validate_addresses(sender, recipients, ccs, bccs)
+
         if reply_to:
             headers['Reply-To'] = reply_to
 
